@@ -1,9 +1,9 @@
 from __future__ import unicode_literals
 import warnings
+from django.core.exceptions import ValidationError
+from mongoengine.errors import ValidationError
 from rest_framework import serializers
 from rest_framework import fields
-from bson import DBRef
-from mongoengine import dereference
 import mongoengine
 from mongoengine.base import BaseDocument
 from django.core.paginator import Page
@@ -11,6 +11,7 @@ from django.db import models
 from django.forms import widgets
 from django.utils.datastructures import SortedDict
 from rest_framework.compat import get_concrete_model
+from .fields import ReferenceField, ListField, EmbeddedDocumentField, DynamicField
 
 
 class MongoEngineModelSerializerOptions(serializers.ModelSerializerOptions):
@@ -19,7 +20,7 @@ class MongoEngineModelSerializerOptions(serializers.ModelSerializerOptions):
     """
     def __init__(self, meta):
         super(MongoEngineModelSerializerOptions, self).__init__(meta)
-        self.validations = getattr(meta, 'related_model_validations', {})
+        self.depth = getattr(meta, 'depth', 5)
 
 
 class MongoEngineModelSerializer(serializers.ModelSerializer):
@@ -28,18 +29,6 @@ class MongoEngineModelSerializer(serializers.ModelSerializer):
     """
     _options_class = MongoEngineModelSerializerOptions
 
-    def validate_related_field(self, attrs, source, object_type):
-        """
-        Validate related model
-        """
-        value = attrs[source]
-
-        try:
-            object_type.objects.get(pk=value)
-        except object_type.DoesNotExist:
-            raise serializers.ValidationError(object_type.__name__ + ' with PK ' + value + ' does not exists.')
-        return attrs
-
     def perform_validation(self, attrs):
         """
         Rest Framework built-in validation + related model validations
@@ -52,12 +41,11 @@ class MongoEngineModelSerializer(serializers.ModelSerializer):
             if self.partial and source not in attrs:
                 continue
 
-            # Related Model Validations
-            if field_name in self.opts.validations:
+            if field_name in attrs and hasattr(field, 'model_field'):
                 try:
-                    self.validate_related_field(attrs, source, self.opts.validations[field_name])
-                except serializers.ValidationError as err:
-                    self._errors[field_name] = self._errors.get(field_name, []) + list(err.messages)
+                    field.model_field.validate(attrs[field_name])
+                except ValidationError as err:
+                    self._errors[field_name] = str(err)
 
             try:
                 validate_method = getattr(self, 'validate_%s' % field_name, None)
@@ -116,11 +104,26 @@ class MongoEngineModelSerializer(serializers.ModelSerializer):
 
         return ret
 
+    def get_dynamic_fields(self, obj):
+        dynamic_fields = {}
+        if obj and obj._dynamic:
+            for key, value in obj._dynamic_fields.items():
+                dynamic_fields[key] = self.get_field(value)
+        return dynamic_fields
+
     def get_field(self, model_field):
         kwargs = {}
 
-        if model_field.required:
-            kwargs['required'] = False
+        if model_field.__class__ in (mongoengine.ReferenceField, mongoengine.EmbeddedDocumentField,
+                                     mongoengine.ListField, mongoengine.DynamicField):
+            kwargs['model_field'] = model_field
+            kwargs['depth'] = self.opts.depth
+
+        if not model_field.__class__ == mongoengine.ObjectIdField:
+            kwargs['required'] = model_field.required
+
+        if model_field.__class__ == mongoengine.EmbeddedDocumentField:
+            kwargs['document_type'] = model_field.document_type
 
         if model_field.default:
             kwargs['required'] = False
@@ -140,7 +143,10 @@ class MongoEngineModelSerializer(serializers.ModelSerializer):
             mongoengine.FileField: fields.FileField,
             mongoengine.ImageField: fields.ImageField,
             mongoengine.ObjectIdField: fields.Field,
-            mongoengine.ReferenceField: fields.CharField,
+            mongoengine.ReferenceField: ReferenceField,
+            mongoengine.ListField: ListField,
+            mongoengine.EmbeddedDocumentField: EmbeddedDocumentField,
+            mongoengine.DynamicField: DynamicField
         }
 
         attribute_dict = {
@@ -162,60 +168,23 @@ class MongoEngineModelSerializer(serializers.ModelSerializer):
         except KeyError:
             return fields.ModelField(model_field=model_field, **kwargs)
 
-    def transform_object(self, obj, fields, depth):
-        """
-        Models to natives
-        Recursion for embedded models
-        """
-        object_data = obj._data
-        counter = 0
-        emb_obj_count = 0
-
-        for field in fields:
-            if issubclass(object_data[field].__class__, DBRef) or issubclass(object_data[field].__class__, mongoengine.Document):
-                emb_obj_count += 1
-
-        for field in fields:
-            if depth == 0:
-                    object_data = unicode(object_data['id'])
-                    break
-            elif issubclass(object_data[field].__class__, DBRef):
-                object_data = dereference.DeReference().__call__(object_data)
-                if counter < depth*emb_obj_count:
-                    counter += 1
-                    object_data[field] = self.transform_object(object_data[field], object_data[field]._fields, depth-1)
-                else:
-                    object_data[field] = unicode(object_data[field].pk)
-            elif issubclass(object_data[field].__class__, mongoengine.Document):
-                if counter < depth*emb_obj_count:
-                    counter += 1
-                    object_data[field] = self.transform_object(object_data[field], object_data[field]._fields, depth-1)
-                else:
-                    object_data[field] = unicode(object_data[field].pk)
-            else:
-                object_data[field] = unicode(object_data[field])
-
-        return object_data
-
     def to_native(self, obj):
         """
         Rest framework built-in to_native + transform_object
         """
         ret = self._dict_class()
         ret.fields = self._dict_class()
-        depth = self.opts.depth
 
-        for field_name, field in self.fields.items():
+        #Dynamic Document Support
+        dynamic_fields = self.get_dynamic_fields(obj)
+        all_fields = dict(dynamic_fields, **self.fields)
+
+        for field_name, field in all_fields.items():
             if field.read_only and obj is None:
                 continue
             field.initialize(parent=self, field_name=field_name)
             key = self.get_field_key(field_name)
             value = field.field_to_native(obj, field_name)
-            #Support for custom fields, check key exists on default fields
-            if key in obj._data:
-                #Call transform_object if field is a related model
-                if issubclass(obj._data[key].__class__, mongoengine.Document) or isinstance(obj._data[key], DBRef):
-                    value = self.transform_object(obj._data[key], value, depth)
             #Override value with transform_ methods
             method = getattr(self, 'transform_%s' % field_name, None)
             if callable(method):
@@ -231,6 +200,9 @@ class MongoEngineModelSerializer(serializers.ModelSerializer):
 
         if data is not None or files is not None:
             attrs = self.restore_fields(data, files)
+            for key in data.keys():
+                if key not in attrs:
+                    attrs[key] = data[key]
             if attrs is not None:
                 attrs = self.perform_validation(attrs)
         else:
@@ -261,4 +233,4 @@ class MongoEngineModelSerializer(serializers.ModelSerializer):
             else:
                 self._data = self.to_native(obj)
 
-        return self._data
+        return self._data
\ No newline at end of file
